import psycopg2

import dbpatch.options
from dbpatch.log import LogFactory
import dbpatch.connection
from dbpatch.context import Context


class PostgreSQLConnection(dbpatch.connection.DbApi2Connection):
    
    NAME = 'postgresql'

    def __init__(self):
        dbpatch.connection.DbApi2Connection.__init__(self)
        self._log = LogFactory().get_logger(PostgreSQLConnection.__name__)
        self._options = Context().options.get_options()

    def connect(self):
        pg_args = {'host'    : self._options[dbpatch.options.DB_HOST],
                   'user'    : self._options[dbpatch.options.DB_USER],
                   'password': self._options[dbpatch.options.DB_PASS],
                   }
        if Context().current_db:
            pg_args['database'] = Context().current_db
        else:
            pg_args['database'] = self.get_internal_database()

        if self._options[dbpatch.options.DB_PORT]:
            pg_args['port'] = self._options[dbpatch.options.DB_PORT]

        self._connection = psycopg2.connect(**pg_args)

    def get_internal_database(self):
        return 'postgres'

    def get_all_databases(self):
        Context().current_db = self.get_internal_database()
        self.connect()
        self.execute(
            'SELECT datname FROM pg_database WHERE datistemplate = false')
        dbs = []
        for row in self.get_rows():
            dbs.append(row[0])

        return dbs


def register():
    dbpatch.connection.ConnectionFactory.register_connection(
        PostgreSQLConnection)

